/*
*   $Id: asm.c,v 1.1.1.1 2001/12/25 22:24:42 corinna Exp $
*
*   Copyright (c) 2000-2001, Darren Hiebert
*
*   This source code is released for free distribution under the terms of the
*   GNU General Public License.
*
*   This module contains functions for generating tags for assembly language
*   files.
*/

/*
*   INCLUDE FILES
*/
#include "general.h"	/* must always come first */

#include <string.h>

#include "parse.h"
#include "read.h"
#include "vstring.h"

/*
*   DATA DEFINITIONS
*/
typedef enum {
    K_DEFINE, K_LABEL, K_MACRO
    /*added by jingtao */, 
    K_PROC, K_CLASS, K_STRUCT,
    K_DATA
} asmKind;

/* indexed by asmKind */
static kindOption AsmKinds [] = {
    { TRUE, 't', "define", "defines (names assigned a specified value)"},
    { TRUE, 'l', "label", "labels (names assigned an address)"},
    { TRUE, 'm', "macro", "macros"}
    /*added by jingtao*/,
    {TRUE, 'p', "proc", "sub procedure"},
    {TRUE, 'c', "class", "masm class"},
    {TRUE, 's', "structure", "masm structure"},
    {TRUE, 'd', "data", "data definition"}
};

/*
*   FUNCTION DEFINITIONS
*/

/* Algorithm adapted from from GNU etags.
 * By Bob Weiner, Motorola Inc., 4/3/94
 * Unix and microcontroller assembly tag handling
 * look for '^ [a-zA-Z_.$] [a-zA_Z0-9_.$]*[: ^I^J]'
 */
static void findAsmTags (void)
{
    vString *name = vStringNew ();
    vString *name2 = vStringNew ();
    const unsigned char *line;

    while ((line = fileReadLine ()) != NULL)
    {
	const unsigned char *cp = line;
	int c = *cp;
	while (isspace (c) || c == '\t')
	    c = *++cp;

	/*  If first char is alphabetic or one of [_.$], test for colon
	 *  following identifier.
	 */
	if (isalpha (c) || c == '_' || c == '.' || c == '$')
	{
	    vStringPut (name, c);
	    c = *++cp;
	    while (isalnum (c) || c == '_' || c == '.' || c == '$')
	    {
		vStringPut (name, c);
		c = *++cp;
	    }
	    vStringTerminate (name);

	    while (isspace (c) || c =='\t')
		c = *++cp;
	    if (c == ':')
		makeSimpleTag (name, AsmKinds, K_LABEL);
	    else 
	    {
		while (isalnum (c) || c == '_')
		{
		    vStringPut (name2, c);
		    c = *++cp;
		}
		vStringTerminate (name2);
		if (vStringLength (name2) > 0)
		{
		    if (stricmp (vStringValue (name2), "=") == 0 ||
			stricmp (vStringValue (name2), "equ") == 0 ||
			stricmp (vStringValue (name2), "textequ") == 0 ||
			stricmp (vStringValue (name2), "typedef") == 0)
			    makeSimpleTag (name, AsmKinds, K_DEFINE);
		    else if (strcmp (vStringValue (name), ".macro") == 0)
		    {
			makeSimpleTag (name2, AsmKinds, K_MACRO);
		    }
		    else if (stricmp (vStringValue (name2), "macro") == 0)
		    {
			makeSimpleTag (name, AsmKinds, K_MACRO);
		    }
		    else if (stricmp(vStringValue (name2), "proc") == 0)
		    {
			makeSimpleTag (name, AsmKinds, K_PROC);
		    }
		    else if (stricmp(vStringValue (name2), "struct") == 0)
		    {
			makeSimpleTag (name, AsmKinds, K_STRUCT);
		    }
		    else if (stricmp(vStringValue (name2), "db") == 0 ||
			     stricmp(vStringValue (name2), "dw") == 0 ||
			     stricmp(vStringValue (name2), "dd") == 0 ||
			     stricmp(vStringValue (name2), "byte") == 0 ||
			     stricmp(vStringValue (name2), "word") == 0 ||
			     stricmp(vStringValue (name2), "dword") == 0 ||
			     stricmp(vStringValue (name2), "label") == 0) 
		    {
			makeSimpleTag (name, AsmKinds, K_DATA);
		    }
		    else if (strcmp(vStringValue (name), "GLOBAL") == 0 ||
			     strcmp(vStringValue (name), "STATICVAR") == 0) 
			makeSimpleTag (name2, AsmKinds, K_DATA);

		    else if (stricmp (vStringValue (name), "class") == 0 ||
			     stricmp (vStringValue (name), "cmethod") == 0)
		    {
			makeSimpleTag (name2, AsmKinds, K_CLASS);
		    }	    
		    /*added by Jingtao,end here.*/
		}
		vStringClear (name2);
	    }
	    vStringClear (name);
	}
    }
}
    
extern parserDefinition* AsmParser (void)
{
    static const char *const extensions [] = { "asm", "s", "S", 
	/*added by jingtao*/ "inc", "ASM", "INC", 
        /*addedby jingtao ended here*/ NULL };
    parserDefinition* def = parserNew ("Asm");
    def->kinds      = AsmKinds;
    def->kindCount  = KIND_COUNT (AsmKinds);
    def->extensions = extensions;
    def->parser     = findAsmTags;
    return def;
}

/* vi:set tabstop=8 shiftwidth=4: */
